autowatch = 1;
outlets = 1;

var RenderContext = "project";

var render = new JitterObject("jit.gl.render", RenderContext);
render.lightin_enable = 1;

var handle = new JitterObject("jit.gl.handle", RenderContext);
handle.inherit_transform = 1;

var sketch = new JitterObject("jit.gl.sketch", RenderContext);
sketch.immediate = 1;

var shader = new JitterObject("jit.gl.shader", RenderContext);
shader.file = "light.projection.cg.jxs";
shader.name = "beamer" + Math.floor(Math.random()*100.);

var texture_mat = new Array();

var projection_pos = [1., 1., 1.];
var camera_pos = [0., 1., 4.];
var projector_lens_angle = 45;

var ground = new JitterObject("jit.gl.gridshape", RenderContext);
ground.shape = "plane";
ground.position = [0, -1., 1.];
ground.rotate = [90, -1, 0, 0];
ground.scale = [10., 10., 10.];
ground.lighting_enable = 1;
ground.blend_enable = 1;
ground.color = [1., 1., 1., 1.];
ground.texture = "movie";
ground.shader = shader.name;
ground.auto_material = 0;
ground.automatic = 0;

var sphere = new JitterObject("jit.gl.gridshape", RenderContext);
sphere.shape = "sphere";
sphere.lighting_enable = 1;
sphere.blend_enable = 1;
sphere.color = [1., 1., 1., 1.];
sphere.texture = "movie";
sphere.shader = shader.name;
sphere.auto_material = 0;
sphere.automatic = 0;

var objects = new Array()
objects[objects.length] = ground;
objects[objects.length] = sphere;


function camera(vals)
{
	camera_pos = arrayfromargs(arguments);
}

function projector(vals)
{
	projection_pos = arrayfromargs(arguments);
}

function lens_angle(val)
{
	projector_lens_angle = val;
}

function projectiveTextureTransform()
{
	var matrix;
	var lookat = render.lookat;
	var up = render.up;
	
	sketch.glmatrixmode("modelview");
	sketch.glpushmatrix();

		sketch.glloadidentity();
		
		//texture scale-bias matrix
		sketch.gltranslate(0.5, 0.5, 0.5);
		sketch.glscale(0.5, 0.5, 0.5);
		
		//light projection matrix (lens angle, aspect ratio, near clip, far clip)
		sketch.gluperspective(projector_lens_angle, 320./240., 0.1, 100.);

		//light view matrix
		sketch.glulookat(projection_pos[0], projection_pos[1], projection_pos[2],
							lookat[0], lookat[1], lookat[2],
							up[0], up[1], up[2]);
		
		//get the matrix
		matrix = sketch.glget("modelview_matrix");

	sketch.glpopmatrix();
	
	return matrix;
}

function modelToWorldTransform(object)
{
	var matrix;
	var position = object.position;
	var rotate = object.rotate;
	var scale = object.scale;
	
	//get camera's view matrix
	sketch.glmatrixmode("modelview");
	sketch.glpushmatrix();
		sketch.glloadidentity();
		sketch.gltranslate(position[0], position[1], position[2]);
		sketch.glrotate(rotate[0], rotate[1], rotate[2], rotate[3]);
		sketch.glscale(scale[0], scale[1], scale[2], scale[3]);
		
		matrix = sketch.glget("modelview_matrix");
	sketch.glpopmatrix();
	
	return matrix;
}

function erase()
{
	render.position = handle.position;
	render.rotate = handle.rotate;

	render.erase();
	render.light_position = projection_pos;

	//calculate texture matrix
	texture_mat = projectiveTextureTransform();
	shader.param("lightToTex", texture_mat);
	
	render.camera = camera_pos;

	for(var i=0; i < objects.length; i++) {
		var model_to_world = modelToWorldTransform(objects[i]);
		shader.param("modelToWorld", model_to_world);
		objects[i].draw();
	}
}

function bang()
{	
	render.drawclients();
	render.swap()
}

function print4x4_mat(mat)
{
		post(mat[0].toPrecision(4) + "\t\t" +  mat[4].toPrecision(4) + "\t\t" + mat[8].toPrecision(4) + "\t\t" + mat[12].toPrecision(4) + "\n");
		post(mat[1].toPrecision(4) + "\t\t" +  mat[5].toPrecision(4) + "\t\t" + mat[9].toPrecision(4) + "\t\t" + mat[13].toPrecision(4) + "\n");
		post(mat[2].toPrecision(4) + "\t\t" +  mat[6].toPrecision(4) + "\t\t" + mat[10].toPrecision(4) + "\t\t" + mat[14].toPrecision(4) + "\n");
		post(mat[3].toPrecision(4) + "\t\t" +  mat[7].toPrecision(4) + "\t\t" + mat[11].toPrecision(4) + "\t\t" + mat[15].toPrecision(4) + "\n");
}

function print4x4_matx(mat)
{
		post(mat[0] + "\t\t" +  mat[4] + "\t\t" + mat[8] + "\t\t" + mat[12] + "\n");
		post(mat[1] + "\t\t" +  mat[5] + "\t\t" + mat[9] + "\t\t" + mat[13] + "\n");
		post(mat[2] + "\t\t" +  mat[6] + "\t\t" + mat[10] + "\t\t" + mat[14] + "\n");
		post(mat[3] + "\t\t" +  mat[7] + "\t\t" + mat[11] + "\t\t" + mat[15] + "\n");
}